# -*- coding: utf-8 -*-

from socialoauth.sites.base import OAuth2


class NetEase(OAuth2):
    AUTHORIZE_URL = 'https://api.t.163.com/oauth2/authorize'
    ACCESS_TOKEN_URL = 'https://api.t.163.com/oauth2/access_token'
    
    NETEASE_API_URL_PREFIX = 'https://api.t.163.com/'
    
    def build_api_url(self, url):
        return '%s%s' % (self.NETEASE_API_URL_PREFIX, url)
    
    def build_api_data(self, **kwargs):
        data = {
            'access_token': self.access_token
        }
        data.update(kwargs)
        return data
    
    def parse_token_response(self, res):
        self.uid = res['uid']
        self.access_token = res['access_token']
        self.expires_in = res['expires_in']
        self.refresh_token = res['refresh_token']
        
        res = self.api_call_get('users/show.json')
        
        self.name = res['name']
        self.avatar = res['profile_image_url']
        self.avatar_large = ""
        
        
        



# -*- coding: utf-8 -*-

import base64

from socialoauth.sites.base import OAuth2


class Sohu(OAuth2):
    AUTHORIZE_URL = 'https://api.t.sohu.com/oauth2/authorize'
    ACCESS_TOKEN_URL = 'https://api.t.sohu.com/oauth2/access_token'
    
    SOHU_API_URL_PREFIX = 'https://api.t.sohu.com/'
    
    def __init__(self):
        super(Sohu, self).__init__()
        self.CLIENT_SECRET = base64.b64encode(self.CLIENT_SECRET)
    
    
    @property
    def authorize_url(self):
        url = super(Sohu, self).authorize_url
        return '%s&scope=basic&wrap_client_state=socialoauth' % url
        
    
    
    def build_api_url(self, url):
        return '%s%s' % (self.SOHU_API_URL_PREFIX, url)
    
    def build_api_data(self, **kwargs):
        data = {
            'access_token': self.access_token
        }
        data.update(kwargs)
        return data
    
    def parse_token_response(self, res):
        self.access_token = res['access_token']
        self.expires_in = res['expires_in']
        self.refresh_token = res['refresh_token']
        
        res = self.api_call_get('users/show.json')
        
        self.uid = res['id']
        self.name = res['screen_name']
        self.avatar = res['profile_image_url']
        self.avatar_large = ""
        
        
        



# -*- coding: utf-8 -*-

class SocialOAuthException(Exception):
    pass


class SocialConfigError(Exception):
    pass



class SocialAPIError(SocialOAuthException):
    """Occurred when doing API call"""
    def __init__(self, site_name, url, error_msg, *args):
        self.site_name = site_name
        self.url = url
        self.error_msg = error_msg
        SocialOAuthException.__init__(self, error_msg, *args)



# -*- coding: utf-8 -*-

from socialoauth.sites.base import OAuth2


class Baidu(OAuth2):
    AUTHORIZE_URL = 'https://openapi.baidu.com/oauth/2.0/authorize'
    ACCESS_TOKEN_URL = 'https://openapi.baidu.com/oauth/2.0/token'
    
    BAIDU_API_URL_PREFIX = 'https://openapi.baidu.com/rest/2.0/'
    SMALL_IMAGE = 'http://tb.himg.baidu.com/sys/portraitn/item/'
    LARGE_IMAGE = 'http://tb.himg.baidu.com/sys/portrait/item/'
    
    RESPONSE_ERROR_KEY = 'error_code'
    
    def build_api_url(self, url):
        return '%s%s' % (self.BAIDU_API_URL_PREFIX, url)
    
    def build_api_data(self, **kwargs):
        data = {
            'access_token': self.access_token,
        }
        data.update(kwargs)
        return data
    
    def parse_token_response(self, res):
        self.access_token = res['access_token']
        self.expires_in = res['expires_in']
        self.refresh_token = res['refresh_token']
        
        res = self.api_call_get('passport/users/getLoggedInUser')
        
        self.uid = res['uid']
        self.name = res['uname']
        self.avatar = '%s%s' % (self.SMALL_IMAGE, res['portrait'])
        self.avatar_large = '%s%s' % (self.LARGE_IMAGE, res['portrait'])



# -*- coding: utf-8 -*-

from socialoauth.exception import SocialConfigError


version_info = (0, 2, 4)
VERSION = __version__ = '.'.join( map(str, version_info) )


class Settings(object):
    """
    This class hold the sites settings.
    User define OAuth2 configs, e.g. site name, oauth class, client_id, client_secret
    in settings file. This Class will parse and store it.
    
    When a site OAuth2 class instantiated, It will load settings from there
    """
    def __init__(self):
        self._configed = False
        self._sites_id_name_table = {}
        # {1: 'renren', 2: 'douban'...}
        self._sites_name_class_table = {}
        # {'renren': 'socialoauth.sites.renren.RenRen',...}
        self._sites_class_config_table = {}
        # {'socialoauth.sites.renren.RenRen': {...}, ...}
        
        
    def __getitem__(self, name):
        """Get OAuth2 Class by it's setting name"""
        if not self._configed:
            raise SocialConfigError("No configure")
        
        try:
            return self._sites_name_class_table[name]
        except KeyError:
            raise SocialConfigError("No settings for site: %s" % name)
        
        
        
    def config(self, settings):
        """Call This method when application start"""
        for k, v in settings.iteritems():
            _module_class, _site_id, _site_name_zh, _site_config = v
            if _site_id in self._sites_id_name_table:
                raise SocialConfigError(
                    "Duplicate site id %d with site name %s" % (_site_id, k)
                )
            self._sites_id_name_table[_site_id] = k
            
            self._sites_name_class_table[k] = _module_class
            self._sites_class_config_table[_module_class] = {
                'site_name': k,
                'site_name_zh': _site_name_zh,
                'site_id': _site_id,
            }
            
            for _k, _v in _site_config.iteritems():
                self._sites_class_config_table[_module_class][_k.upper()] = _v
                
        self._configed = True
        
    
    def load_config(self, module_class_name):
        """
        OAuth2 Class get it's settings at here.
        Example:
            from socialoauth import socialsites
            class_key_name = Class.__module__ + Class.__name__
            settings = socialsites.load_config(class_key_name)
        """
        return self._sites_class_config_table[module_class_name]
        
        
    def list_sites(self):
        return self._sites_class_config_table.keys()
    
    
    def get_site_name_by_id(self, site_id):
        try:
            return self._sites_id_name_table[site_id]
        except KeyError:
            raise SocialConfigError("No settings for site id : %d" % site_id)
        
    def get_site_class_by_id(self, site_id):
        site_name = self.get_site_name_by_id(site_id)
        return self.__getitem__(site_name)
        
        
        
socialsites = Settings()




# -*- coding: utf-8 -*-

from socialoauth.sites.base import OAuth2


class DouBan(OAuth2):
    AUTHORIZE_URL = 'https://www.douban.com/service/auth2/auth'
    ACCESS_TOKEN_URL = 'https://www.douban.com/service/auth2/token'
    
    DOUBAN_API_URL = 'https://api.douban.com'
    
    
    def build_api_url(self, url):
        return '%s%s' % (self.DOUBAN_API_URL, url)
    
    def build_api_data(self, **kwargs):
        return kwargs
    
    
    def http_add_header(self, req):
        """Douban API call must set `access_token` in headers"""
        if getattr(self, 'access_token', None) is None:
            return
        req.add_header('Authorization',  'Bearer %s' % self.access_token)
        
    
    def parse_token_response(self, res):
        self.uid = res['douban_user_id']
        self.access_token = res['access_token']
        self.expires_in = res['expires_in']
        self.refresh_token = res['refresh_token']
        
        res = self.api_call_get('/v2/user/~me')
        
        
        self.name = res['name']
        self.avatar = res['avatar']
        self.avatar_large = ""
        
        



# -*- coding: utf-8 -*-

from socialoauth.sites.base import OAuth2


class Weibo(OAuth2):
    AUTHORIZE_URL = 'https://api.weibo.com/oauth2/authorize'
    ACCESS_TOKEN_URL = 'https://api.weibo.com/oauth2/access_token'
    
    
    def build_api_url(self, url):
        return url
    
    def build_api_data(self, **kwargs):
        data = {
            'access_token': self.access_token
        }
        data.update(kwargs)
        return data
    
    def parse_token_response(self, res):
        self.uid = res['uid']
        self.access_token = res['access_token']
        self.expires_in = res['expires_in']
        self.refresh_token = None
        
        res = self.api_call_get(
            'https://api.weibo.com/2/users/show.json',
            uid=self.uid
        )
        
        self.name = res['name']
        self.avatar = res['profile_image_url']
        self.avatar_large = res['avatar_large']
        
        
        
    def post_status(self, text):
        if isinstance(text, unicode):
            text = text.encode('utf-8')
            
        url = 'https://api.weibo.com/2/statuses/update.json'
        res = self.api_call_post(url, status=text)
        



# -*- coding: utf-8 -*-

from urllib import urlencode
import urllib2
import json
from functools import wraps

from socialoauth.exception import SocialAPIError
from socialoauth import socialsites

HTTP_TIMEOUT = 10


def _http_error_handler(func):
    @wraps(func)
    def deco(self, *args, **kwargs):
        try:
            res = func(self, *args, **kwargs)
        except urllib2.HTTPError as e:
            raise SocialAPIError(self.site_name, e.url, e.read())
        except urllib2.URLError as e:
            raise SocialAPIError(self.site_name, args[0], e.reason)
        
        error_key = getattr(self, 'RESPONSE_ERROR_KEY', None)
        if error_key is not None and error_key in res:
            raise SocialAPIError(self.site_name, args[0], res)
        
        return res
    return deco


class OAuth2(object):
    """
    Base OAuth2 class, Sub class must define the following settings:

    AUTHORIZE_URL    - Asking user to authorize and get token
    ACCESS_TOKEN_URL - Get authorized access token

    And the bellowing should define in settings file

    REDIRECT_URI     - The url after user authorized and redirect to
    CLIENT_ID        - Your client id for the social site
    CLIENT_SECRET    - Your client secret for the social site

    Also, If the Website needs scope parameters, your should add it too.

    SCOPE            - A list type contains some scopes

    Details see: http://tools.ietf.org/html/rfc6749



    SubClass MUST Implement the following three methods:

    build_api_url(self, url)
    build_api_data(self, **kwargs)
    parse_token_response(self, res)
    """

    def __init__(self):
        """Get config from settings.
        class instance will have the following properties:
        
        site_name
        site_id
        REDIRECT_URI
        CLIENT_ID
        CLIENT_SECRET
        """
        key = '%s.%s' % (self.__class__.__module__, self.__class__.__name__)
        configs = socialsites.load_config(key)
        for k, v in configs.iteritems():
            setattr(self, k, v)


    @_http_error_handler
    def http_get(self, url, data, parse=True):
        req = urllib2.Request('%s?%s' % (url, urlencode(data)))
        self.http_add_header(req)
        res = urllib2.urlopen(req, timeout=HTTP_TIMEOUT).read()
        if parse:
            return json.loads(res)
        return res


    @_http_error_handler
    def http_post(self, url, data, parse=True):
        req = urllib2.Request(url, data=urlencode(data))
        self.http_add_header(req)
        res = urllib2.urlopen(req, timeout=HTTP_TIMEOUT).read()
        if parse:
            return json.loads(res)
        return res


    def http_add_header(self, req):
        """Sub class rewiter this function If it's necessary to add headers"""
        pass



    @property
    def authorize_url(self):
        """Rewrite this property method If there are more arguments
        need  attach to the url. Like bellow:

            class NewSubClass(OAuth2):
                @property
                def authorize_url(self):
                    url = super(NewSubClass, self).authorize_url
                    url += '&blabla'
                    return url
        """

        url = "%s?client_id=%s&response_type=code&redirect_uri=%s" % (
            self.AUTHORIZE_URL, self.CLIENT_ID, self.REDIRECT_URI
        )

        if getattr(self, 'SCOPE', None) is not None:
            url = '%s&scope=%s' % (url, '+'.join(self.SCOPE))

        return url



    def get_access_token(self, code, method='POST', parse=True):
        """parse is True means that the api return a json string. 
        So, the result will be parsed by json library.
        Most sites will follow this rule, return a json string.
        But some sites (e.g. Tencent), Will return an non json string,
        This sites MUST set parse=False when call this method,
        And handle the result by themselves.

        This method Maybe raise SocialAPIError.
        Application MUST try this Exception.
        """

        data = {
            'client_id': self.CLIENT_ID,
            'client_secret': self.CLIENT_SECRET,
            'redirect_uri': self.REDIRECT_URI,
            'code': code,
            'grant_type': 'authorization_code'
        }


        if method == 'POST':
            res = self.http_post(self.ACCESS_TOKEN_URL, data, parse=parse)
        else:
            res = self.http_get(self.ACCESS_TOKEN_URL, data, parse=parse)

        self.parse_token_response(res)



    def api_call_get(self, url=None, **kwargs):
        url = self.build_api_url(url)
        data = self.build_api_data(**kwargs)
        return self.http_get(url, data)

    def api_call_post(self, url=None, **kwargs):
        url = self.build_api_url(url)
        data = self.build_api_data(**kwargs)
        return self.http_post(url, data)




    def parse_token_response(self, res):
        """
        Subclass MUST implement this method
        And set the following attributes:

        access_token,
        uid,
        name,
        avatar,
        """
        raise NotImplementedError


    def build_api_url(self, url):
        raise NotImplementedError


    def build_api_data(self, **kwargs):
        raise NotImplementedError




# -*- coding: utf-8 -*-
import os
import sys


from _bottle import Bottle, run, request, response, redirect, static_file


CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.normpath(os.path.join(CURRENT_PATH, '..')))

IMAGE_PATH = os.path.join(CURRENT_PATH, 'images')

from settings import SOCIALOAUTH_SITES
from socialoauth import socialsites
from socialoauth.utils import import_oauth_class
from socialoauth.exception import SocialAPIError

from helper import Session, UserStorage, gen_session_id

socialsites.config(SOCIALOAUTH_SITES)



app = Bottle()


@app.get('/static/images/<filepath:path>')
def static_files(filepath):
    return static_file(filepath, IMAGE_PATH, mimetype='image/png')


@app.get('/')
def index():
    session_id = request.get_cookie('session_id')
    if session_id:
        session = Session()
        data = session.get(session_id)
        uid = data.get('uid', None)
        if uid:
            storage = UserStorage()
            user = storage.get_user(uid)
            
            
            html = """<!DOCTYPE html>
            <html>
                <body>
                <h2>Welcome. %s</h2>
                <p>you from %s, and your social id is %s<p>
                <img src="%s" />
                <p><a href="/logout">Logout</a></p>
                </body>
            </html>""" % (user['name'], user['site_name'], user['uid'], user['avatar'])
            
            return html
    
    if not session_id:
        response.set_cookie('session_id', gen_session_id())
    html = """<html>
    <body><a href="/login">Login</a></body>
    </html>"""
    
    return html




@app.get('/login')
def login():
    def _link(s):
        _s = import_oauth_class(s)()
        if os.path.exists(os.path.join(IMAGE_PATH, _s.site_name + '.png')):
            a_content = '<img src="/static/images/%s.png" />' % _s.site_name
        else:
            a_content = '使用 %s 登录' % _s.site_name_zh
        
        return """<div style="margin: 20px;">
        <a href="%s">%s</a>
        </div>""" % (_s.authorize_url, a_content)
    
    links = map(_link, socialsites.list_sites())
    links = '\n'.join(links)
    
    
    html = """<!DOCTYPE html>
    <html>
        <body>%s</body>
    </html>
    """ % links
    
    return html
    
    
    
@app.get('/account/oauth/<sitename>')
def callback(sitename):
    code = request.GET.get('code')
    if not code:
        # error occurred
        redirect('/oautherror')
    
    s = import_oauth_class(socialsites[sitename])()
    try:
        s.get_access_token(code)
    except SocialAPIError as e:
        # 这里可能会发生错误
        print e.site_name      # 哪个站点的OAuth2发生错误？
        print e.url            # 请求的url
        print e.error_msg      # 由站点返回的错误信息 / urllib2 的错误信息
        raise
    
    # 到这里授权完毕，并且取到了用户信息，uid, name, avatar...
    storage = UserStorage()
    UID = storage.get_uid(s.site_name, s.uid)
    if not UID:
        # 此用户第一次登录，为其绑定一个自身网站的UID
        UID = storage.bind_new_user(s.site_name, s.uid)
        
    
    storage.set_user(
        UID,
        site_name = s.site_name,
        site_id = s.site_id,
        uid = s.uid,
        name = s.name,
        avatar = s.avatar
    )
    
    session_id = request.get_cookie('session_id')
    if not session_id:
        session_id = Session.make_session_id(UID)
    session = Session()
    session.set(session_id, uid=UID)
    response.set_cookie('session_id', session_id)
    
    
    
    redirect('/')
    
    
    
    
@app.get('/logout')
def logout():
    session_id = request.get_cookie('session_id')
    if not session_id:
        redirect('/')
    
    session = Session()
    data = session.get(session_id)
    session.rem(session_id)
    uid = data.get('uid', None)
    if uid:
        # 重置其session_id
        Session.refresh_session_id(uid)
        
    response.set_cookie('session_id', '')
    redirect('/')
    
    
    
@app.get('/oautherror')
def oautherror():
    print 'OAuth Error!'
    redirect('/')
    
    
if __name__ == '__main__':
    run(app, port=8000)



# -*- coding: utf-8 -*-

from socialoauth.sites.base import OAuth2


class RenRen(OAuth2):
    AUTHORIZE_URL = 'https://graph.renren.com/oauth/authorize'
    ACCESS_TOKEN_URL = 'http://graph.renren.com/oauth/token'
    
    RENREN_API_URL = 'https://api.renren.com/restserver.do'
    RESPONSE_ERROR_KEY = 'error_code'
    
    
    def build_api_url(self, *args):
        return self.RENREN_API_URL
    
    
    def build_api_data(self, **kwargs):
        data = {
            'v': 1.0,
            'access_token': self.access_token,
            'format': 'json',
        }
        data.update(kwargs)
        return data
    
    
    
    def parse_token_response(self, res):
        self.uid = res['user']['id']
        self.access_token = res['access_token']
        self.expires_in = res['expires_in']
        self.refresh_token = res['refresh_token']
        
        res = self.api_call_post(method='users.getInfo')
        self.name = res[0]['name']
        self.avatar = res[0]['tinyurl']
        self.avatar_large = res[0]['headurl']
        
        
        
        
    def post_status(self, text):
        if isinstance(text, unicode):
            text = text.encode('utf-8')
        
        res = self.api_call_post(method='status.set', status=text)




# -*- coding: utf-8 -*-

import json
import random
import hashlib




class SingletonGuard(type):
    def __init__(self, name, parent, class_dict):
        super(SingletonGuard, self).__init__(name, parent, class_dict)
        self.instance = None
        
        
    def __call__(self, *args, **kwargs):
        if self.instance is None:
            self.instance = super(SingletonGuard, self).__call__(*args, **kwargs)
        return self.instance
        
        

class UserStorage(object):
    __metaclass__ = SingletonGuard
    
    def __init__(self):
        # 这是自身系统的用户ID，模拟数据库的自增长主键
        self.ID = 0
        # 存储社交网站uid于自身ID的对应关系
        self.table = {}
        # 用户信息
        self.user = {}
        
    def get_uid(self, site_name, site_uid):
        # site_name 是社交网站的 名字
        # site_uid 是此授权用户在此网站的uid
        # 查询此授权用户是否在自身数据库中的UID
        
        return self.table.get(site_name, {}).get(site_uid, None)
        
    
    
    def bind_new_user(self, site_name, site_uid):
        self.ID += 1
        if site_name in self.table:
            self.table[site_name][site_uid] = self.ID
        else:
            self.table[site_name] = {site_uid: self.ID}
            
        return self.ID
    
    
    def get_user(self, inner_uid):
        return self.user[inner_uid]
    
    def set_user(self, inner_uid, **kwargs):
        self.user[inner_uid] = kwargs





def gen_session_id():
    key = '%0.10f' % random.random()
    return hashlib.sha1(key).hexdigest()



class Session(object):
    __metaclass__ = SingletonGuard
    
    uid_session_keys = {}
    
    def __init__(self):
        self._sessions = {}
        
        
    @classmethod
    def make_session_id(cls, uid):
        if uid not in cls.uid_session_keys:
            cls.uid_session_keys[uid] = gen_session_id()
        return cls.uid_session_keys[uid]
    
    
    @classmethod
    def refresh_session_id(cls, uid):
        cls.uid_session_keys[uid] = gen_session_id()
        return cls.uid_session_keys[uid]
        
    
    def get(self, key):
        if key not in self._sessions:
            return {}
        return json.loads(self._sessions[key])
    
    def set(self, key, **kwargs):
        self._sessions[key] = json.dumps(kwargs)
        
        
    def update(self, key, **kwargs):
        s = self.get(key)
        if not s:
            self.set(key, **kwargs)
        else:
            s.update(kwargs)
            self.set(key, **s)
            
            
    def rem(self, key):
        if key in self._sessions:
            del self._sessions[key]



from distutils.core import setup

import socialoauth

packages = [
    'socialoauth',
    'socialoauth.sites',
]


setup(
    name='socialoauth',
    version = socialoauth.VERSION,
    license = 'MIT',
    description = 'Python Package For SNS sites with OAuth2 support',
    long_description = open('README.txt').read(),
    author = 'Wang Chao',
    author_email = 'yueyoum@gmail.com',
    url = 'https://github.com/yueyoum/social-oauth',
    keywords = 'social, oauth, oauth2',
    packages = packages,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Topic :: Internet',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



# -*- coding: utf-8 -*-
import re
import json

from socialoauth.sites.base import OAuth2
from socialoauth.exception import SocialAPIError


QQ_OPENID_PATTERN = re.compile('\{.+\}')

class QQ(OAuth2):
    AUTHORIZE_URL = 'https://graph.qq.com/oauth2.0/authorize'
    ACCESS_TOKEN_URL = 'https://graph.qq.com/oauth2.0/token'
    
    OPENID_URL = 'https://graph.qq.com/oauth2.0/me'
    
    
    @property
    def authorize_url(self):
        url = super(QQ, self).authorize_url
        return '%s&state=socialoauth' % url
    
    
    def get_access_token(self, code):
        super(QQ, self).get_access_token(code, method='GET', parse=False)
        
    
    def build_api_url(self, url):
        return url
    
    def build_api_data(self, **kwargs):
        data = {
            'access_token': self.access_token,
            'oauth_consumer_key': self.CLIENT_ID,
            'openid': self.uid
        }
        data.update(kwargs)
        return data
    
    
    
    def parse_token_response(self, res):
        res = res.split('&')
        res = [_r.split('=') for _r in res]
        res = dict(res)
        
        self.access_token = res['access_token']
        self.expires_in = int(res['expires_in'])
        self.refresh_token = None
        
        res = self.http_get(self.OPENID_URL, {'access_token': self.access_token}, parse=False)
        res = json.loads(QQ_OPENID_PATTERN.search(res).group())
        
        self.uid = res['openid']
        
        _url = 'https://graph.qq.com/user/get_user_info'
        res = self.api_call_get(_url)
        if res['ret'] != 0:
            raise SocialAPIError(self.site_name, _url, res)
        
        
        self.name = res['nickname']
        self.avatar = res['figureurl_1']
        self.avatar_large = res['figureurl_2']
        
